import Utils from "./utils.js";
import readline from "readline";
import DeploySql from './lib/sql.js'

const rl = readline.createInterface({
  input: process.stdin,
  output: process.stdout,
});

export default class BuildTransactions {
  constructor() {}

  getObjectsMessage(file) {
    let sqlObjName = file.path.split('/')[1];
    let verb, fullName;

    try {
      verb = file.type.toLowerCase() === "m" ? "Altering" : "Creating";
      fullName = `${verb} ${sqlObjName}`;
    } catch (e) {
      console.error(e);
      fullName = "ERROR ON NAMING";
    }
    return fullName;
  }

  getDataMessage(f) {
    let fileName = f.split('/');
    return fileName[fileName.length-1];
  }

  async BuildObjectsFile(filesArr, options) {
    const fileInfo = Utils.GetFinishedTransactions(
      await Utils.MakeTransactions(
        filesArr,
        this.getObjectsMessage
      ),
      "object"
    );
    Utils.BuildFile(options.name, fileInfo);
    if (options.server) {
      DeploySql(options)
    }
  }
  async BuildDataFile(filesArr, options) {
    const redshiftReg = new RegExp("[Rr]edshift");
    const redshiftData = [];
    const lmsData = [];
    filesArr.forEach((f) => redshiftReg.test(f.path) ? redshiftData.push(f.path) : lmsData.push(f.path) );
    
    
    if (redshiftData.length === 0 && options.redshift) {
      console.log('No data found');
      process.exit()
    }

    if (lmsData.length === 0 && !options.redshift) {
      console.log('No data found');
      process.exit()
    }

    if (options.redshift && redshiftData.length > 0) {
      const redInfo = await Utils.MakeBuild(
          redshiftData,
          this.getDataMessage
        );
      Utils.BuildFile(`Redshift_${options.name}`, redInfo);
    }

    if (!options.redshift && lmsData.length > 0) {
      const lmsInfo = Utils.GetFinishedTransactions(
        await Utils.MakeTransactions(lmsData, this.getDataMessage, true),
        "data"
      );      
      Utils.BuildFile(options.name, lmsInfo);
      if (options.server) {
        DeploySql(options)
      }
    }
  }
}
