import fs from "fs";
import languageEncoding from "detect-file-encoding-and-language";

export default class Utils {  
  static GetFiles(directory) {
    try {
      if (directory === ".") {
        return fs.readdirSync("./");
      }
      return fs.readdirSync(directory);
    } catch (err) {
      console.error(err);
    }
  }

  static BuildFile(output, fileInfo) {
    if(!fs.existsSync('Builds')) fs.mkdirSync('Builds')
    fs.writeFile('Builds/' + output, fileInfo, (err) =>{
      err ? console.error(err) : console.log(`${output} file has been built\n`);
      process.exit();
    }
    );
  }

  static getDir(dir) {
    try {
      return fs.readdirSync(dir);
    } catch (error) {
      console.warn(`${dir} not a directory`);
      return false;
    }
  }

  static async MakeBuild(filesArr, messageFunc) {
    let stringBuilder = "";
    for (let file of filesArr) {      
      const path = file;
      if (!fs.existsSync(path)) {
        console.warn(`${path} does not exist`);
        continue;
      }
      const fileLoc = `${path}`;
      console.log(fileLoc);      
      const data = fs.readFileSync(fileLoc);
      const dataStr = data.toString("utf8");      
      stringBuilder += "--" + messageFunc(file);
      stringBuilder += dataStr + "\n";
    }
    return stringBuilder;
  }

  static async MakeTransactions(filesArr, messageFunc, isData = false) {
    let transactions = "";    
    for (let file of filesArr) {      
      let path = isData ? file : file?.path;
      if (!fs.existsSync(path)) {
        console.warn(`${path} does not exist`);
        continue;
      }
      const fileLoc = `${path}`;

      console.log(fileLoc);
      let encoding = '';
 
      encoding = (await languageEncoding(fileLoc)).encoding

      const data = fs.readFileSync(fileLoc, {encoding: encoding == 'UTF-8' ? 'utf8':'utf16le'});
      const dataStr = data.toString("utf8");

      const transaction = `
IF @@ERROR <> 0 SET NOEXEC ON
GO
PRINT N'${messageFunc(file)}'
GO
${dataStr.replace(/﻿/g,'')}
GO
          `;
  
        transactions += transaction;
    }
    return transactions;
  }

  static GetFinishedTransactions(transactions, type) {
    return `\
/*

Generated by deploysql tool 
${type === "data" ? "Data" : "Objects"} Script

*/
SET NUMERIC_ROUNDABORT OFF
GO
SET ANSI_PADDING, ANSI_WARNINGS, CONCAT_NULL_YIELDS_NULL, ARITHABORT, QUOTED_IDENTIFIER, ANSI_NULLS ON
GO
SET XACT_ABORT ON
GO
SET TRANSACTION ISOLATION LEVEL Serializable
GO
BEGIN TRANSACTION
GO
${transactions}
${
  type === "object"
    ? `
IF @@ERROR <> 0 SET NOEXEC ON
GO
    EXEC MaintGrantAll
GO`
    : ""
}
IF @@ERROR <> 0 SET NOEXEC ON
GO
COMMIT TRANSACTION
GO
IF @@ERROR <> 0 SET NOEXEC ON
GO
-- This statement writes to the SQL Server Log so SQL Monitor can show this deployment.
IF HAS_PERMS_BY_NAME(N'sys.xp_logevent', N'OBJECT', N'EXECUTE') = 1
BEGIN
    DECLARE @databaseName AS nvarchar(2048), @eventMessage AS nvarchar(2048)
    SET @databaseName = REPLACE(REPLACE(DB_NAME(), N'\', N'\\'), N'"', N'\"')
    SET @eventMessage = N'SQL Compare: { "deployment": { "description": "SQL Compare deployed to ' + @databaseName + N'", "database": "' + @databaseName + N'" }}'
    EXECUTE sys.xp_logevent 55000, @eventMessage
END
GO
DECLARE @Success AS BIT
SET @Success = 1
SET NOEXEC OFF
IF (@Success = 1) PRINT 'The database update succeeded'
ELSE BEGIN
  IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION
  PRINT 'The database update failed'
END
GO

  `;
  }
}
